clear all; clc; close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solves simple DNK model for optimal Ramsey policy                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% assumes nominal interest rate is ordered first in variables
% assumes non-policy shocks are ordered first in shocks 


Tstar                    = 100; % upper bound on optimal policy horizon

% [AA0,AA1,AA2,AA3,r,beta] = get_model_markup;

[AA0,AA1,AA2,AA3,r,beta] = get_model;

[ny,nv]                  = size(AA3);
[A0,A1,A2,A3]            = add_anticipated(AA0,AA1,AA2,AA3,Tstar,r);

offset   = 20;
T        = Tstar+offset;
div      = 1;            % cutoff between explosive and stationary
[IRF,P0] = solve_model(A0,A1,A2,A3,T,div); 
% IRF is of dimension (length of impulse response, number of variables
% including shocks, number of anticipated shocks)

dd = 10;

IRF(1:10,2,dd+1) % shock dd periods in future

%return;

D = NaN(ny*Tstar,Tstar);
O = zeros(ny*Tstar,ny*Tstar);
B = NaN(ny*Tstar,1);

bigbeta = (beta.^([0:Tstar-1]));

% period loss function
W                = zeros(ny,ny);
output_weight    = 0.0255/6;    % 1/20;
inflation_weight = 0;
change_weight    = 1;
W(2,2) = output_weight;
W(3,3) = inflation_weight;
W(4,4) = change_weight;


% load some baseline
% load_baseline;
% baseline = baseline';

load('baseline_fbl.mat');

% shocksize is not used, if baseline is used to construct B matrix 
shocksize = -0.0165;  %-7/400;

% Create D matrix linking observables to anticipated shocks 
% Create B matrix of baseline projection given some shock
% Create O matrix containing welfare weights
for  n = 1:ny
     D( (n-1)*Tstar+1 : n*Tstar ,:) = squeeze(IRF(1:Tstar,n,nv+1:end));
     B( (n-1)*Tstar+1 : n*Tstar ,:) = baseline_fbl(1:Tstar,n);  %
     % B( (n-1)*Tstar+1 : n*Tstar ,:) = squeeze(IRF(1:Tstar,n,1))*shocksize; 
     O( (n-1)*Tstar+1 : n*Tstar ,(n-1)*Tstar+1 : n*Tstar) = diag(bigbeta).*W(n,n)*eye(Tstar);  
end


% Work out optimal weights on anticipated shocks - (absent ZLB)

XX = (-(B'*O*D)*pinv(D'*O*D))';

options = optimset('MaxIter',10^4,'LargeScale','off','Display','Iter','TolFun',10^(-12));

A = -D(1:Tstar,:);
b = B(1:Tstar,1) - (beta-1)*ones(Tstar,1);
b = B(1:Tstar,1) + log(1/beta)*ones(Tstar,1);

problem.H      = D'*O*D;        % Symmetric matrix
problem.f      = (B'*O*D)';     % Vector
problem.Aineq  = A;             % Matrix for linear inequality constraints
problem.bineq  = b;             % Vector for linear inequality constraints
problem.Aeq    = [];            % Matrix for linear equality constraints
problem.beq    = [];            % Vector for linear equality constraints
problem.lb	   = [];            % Vector of lower bounds
problem.ub	   = [];            % Vector of upper bounds
problem.x0     = ones(Tstar,1); % Initial point for x
problem.solver = 'quadprog';    % solver 
problem.options = options;      % Options structure created with optimset

[X,fval,exitflag,output,lambda] = quadprog(problem);

if exitflag~=1
    warning('solution not admissable');
end

% X(end-10:end) = 0;

% max(abs((X-XX)))

% These are the impulse responses under optimal policy 
IRF2 = reshape(B+D*X,Tstar,n);


NAMES = [{'interest rate','output gap','inflation','change in infaltion','real rate gap','natural rate shock'}];

% Construct responses to joint shock: structural + anticipated
%-------------------------------------------------------------------------
Malpha = squeeze(IRF(1:Tstar,:,nv+1))*X(1);
for p = 2 : Tstar;
   Malpha = Malpha + squeeze(IRF(1:Tstar,:,p+nv))*X(p);
end
joint_IRF            = squeeze(IRF(:,:,1))*shocksize;
joint_IRF(1:Tstar,:) = joint_IRF(1:Tstar,:) + Malpha; 


% Plot figures
%-------------------------------------------------------------------------
c  = 100;
KK = 40;
index = [1,2,3,5];
figure('Name','Commitment Responses (1)','NumberTitle','off')
for j = 1:4
  %  subplot(2,2,j), plot(c*joint_IRF(1:KK,j),'-*b'); axis tight; hold on
    subplot(2,2,j), plot(c*IRF2(1:KK,index(j)),'-*r'); axis tight; hold on;
                    plot(c*baseline_fbl(1:KK,index(j)),'-b'); axis tight; hold off;

    title(NAMES(index(j)));
end
%suptitle('with anticipated shocks')

%orient landscape
print('-depsc','Ramsey_inertia');

IRF_ramsey = IRF2;

save IRF_ramsey IRF_ramsey

DZLB = sum(lambda.ineqlin>sqrt(eps));

fprintf('\n Duration at ZLB is %g quarters \n', DZLB);
fprintf('\n Inflation at the lift off date %g \n',  IRF2(DZLB+1,3)*400);
fprintf('\n Output gap at the lift off date %g \n', IRF2(DZLB+1,2)*100);

return;
% compare with level-k
load IRF2

k = 80;
 diff = max(max(abs(IRF_ramsey(1:end-k,2:3)-IRF2(1:end-k,2:3))))
 
return;

% Compare to optimal targeting criterion
% -----------------------------------------
[A0,A1,A2,A3] = get_model_markup_ramsey;
div           = 1;
[IRF,P0]      = solve_model(A0,A1,A2,A3,T,div);

display('Impulse responses from Targeting Criterion');
IRF(1:10,1:ny)

display('Impulse responses from Shock Procedure');
joint_IRF(1:10,1:ny)
