clear all; clc; close all;
% -------------------------------------------------------------------------
% Here we solve the backward and forward looking equations with level-k
% thinking.  Model includes a taylor rule that accounts for the ZLB and a
% large natural rate shock at time T=1
% -------------------------------------------------------------------------

global beta omega sigma phi_pi phi_y varphi zlb T
 
% dynamic variables
T    = 1000;    % total number of iterations 
k    = 8;    % number of levels 

% global paremeters
beta   = 0.99;
kappa  = .0255;    
omega = 0;
sigma  = 1;
phi_pi = 1.5;
phi_y  = 0.5;
options  = optimset('Display','Off','TolFun',1e-12);
x_start  = .5;
varphi = fsolve(inline('P1 - (1-P2*x)*(1-x)/x*(P3 + inv(P4))',4),x_start,options,kappa,beta,omega,sigma);

% important parameters
rho_nr = 0.9;          % shock persistance
eps_nr = -0.015;       % natural rate shock
zlb = -log(1/beta);      % zlb

% initialize matrices
y_tr    = zeros(T,k+1);   %rows are quarters, t, columns are levels, K starting at level 0
p_tr    = zeros(T,k+1);
ii_tr   = zeros(T,k+1);  % taylor rule
r       = zeros(T,1);   % natural rate

% create natural rate vector 
r(1,1) = 0;
r(2,1) = eps_nr;      % large shock appears at time T=1
for t = 3:T
    r(t,1) = rho_nr*r(t-1,1);
end

% -------------------------------------------------------------------------
% Obtain REE solution
% Order of vars : 'x','pi','i','r_nat'
% -------------------------------------------------------------------------
UD = 0;   % solve ree solution with no ueber discouting
UDP = 0;
[IRF_REE,IRF_DL]   = FBL_ree(eps_nr,UD,UDP);
y_belief  = IRF_REE(1,:)';
p_belief  = IRF_REE(2,:)';
ii_belief = IRF_REE(3,:)';

% initialize prior beliefs
y_tr(2:501,1)  = y_belief;
p_tr(2:501,1)  = p_belief;
ii_tr(2:501,1) = ii_belief;

% -------------------------------------------------------------------------
% Compute Level-K Experiment- solve model with taylor rule
% first column of matrix is level-0, first row is time t=0
% -------------------------------------------------------------------------
for level = 2:k+1
    for index = 2:T-1  %%first row is time t = 0, so start recursion at time t=1

        [sol] = fonc_FBL(index,level,y_tr,p_tr,r);
        y_tr(index,level)   = sol(1);
        p_tr(index,level)   = sol(2);
        ii_tr(index,level)  = sol(3);

    end 
end

%--------------------------------------------------------------------------
% Delayed liftoff- 1:10 periods 
%--------------------------------------------------------------------------
for extra_periods = 1:10
    % initialize cells
    y_dl{extra_periods}    = zeros(T,k+1);   %rows are quarters, t, columns are levels, K starting at level 0
    p_dl{extra_periods}    = zeros(T,k+1);
    ii_dl{extra_periods}   = zeros(T,k+1);   % taylor rule
  
    % set initial beliefs
    y_dl{extra_periods}(2:501,1)  = y_belief;
    p_dl{extra_periods}(2:501,1)  = p_belief;
    ii_dl{extra_periods}(2:501,1) = ii_belief;
    
    for level = 2:k+1
        for index = 2:T-1  %%first row is time t = 0, so start recursion at time t=1

            [sol] = fonc_DL(index,level,extra_periods,ii_tr,y_dl{extra_periods},p_dl{extra_periods},r);
            y_dl{extra_periods}(index,level)  = sol(1);
            p_dl{extra_periods}(index,level)  = sol(2);
            ii_dl{extra_periods}(index,level) = sol(3);

        end 
    end
end

%--------------------------------------------------------------------------
% Produce Plots
%--------------------------------------------------------------------------
close all 
level = 2;

cmap = autumn(11);

call_level = level + 1;
FS=20;
f = figure(1)
plot(100*y_tr(2:21,call_level),'-*b');axis tight;hold on;
for j = 2:2:extra_periods
    plot(100*y_dl{j}(2:21,call_level),'Color',cmap(j,:),'LineWidth',j*1/5);axis tight;hold on;
end
hold off;
h = title('Output Gap');set(h,'FontSize',FS);
h = legend('Taylor','Location','NorthEast');set(h,'FontSize',FS);legend('boxoff');
xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print(f,strcat('DL_FBL_level',num2str(level),'_output'),'-depsc');

f = figure(2)
plot(400*p_tr(2:21,call_level),'-*b');axis tight;hold on;
for j = 2:2:extra_periods
    plot(400*p_dl{j}(2:21,call_level),'Color',cmap(j,:),'LineWidth',j*1/5);axis tight;hold on;
end
hold off;
h = title('Inflation');set(h,'FontSize',FS);
h = legend('Taylor','Location','NorthEast');set(h,'FontSize',FS);legend('boxoff');
xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print(f,strcat('DL_FBL_level',num2str(level),'_infl'),'-depsc');

f = figure(3)
plot(400*(ii_tr(2:21,call_level) - zlb),'-*b');axis tight;hold on;
for j = 2:2:extra_periods
    plot(400*(ii_dl{j}(2:21,call_level) - zlb),'Color',cmap(j,:),'LineWidth',j*1/5);axis tight;hold on;
end
hold off;
h = title('Policy Rate (in levels)');set(h,'FontSize',FS);
h = legend('Taylor','Location','SouthEast');set(h,'FontSize',FS);legend('boxoff');
xt = get(gca, 'XTick'); %change size of tick mark values
set(gca, 'FontSize', FS);
print(f,strcat('DL_FBL_level',num2str(level),'_policyrate'),'-depsc');