function [H1S,H2S,H3S,H4S,F1S,F2S,F3S,F4S,VyS,VyxS,VxS,VycS,VxcS,MUS] = ... 
ZLB_discretion(A0,A1,A2,A3,A5,A6,S,s,J_T,J_C1,J_C2,J_C3,W,ZZ,RR,H1,H2,H4,V_x,V_y,V_yx,V_yc,V_xc,beta,t,step)

% INPUT:   
%          A0,A1,A2,A3,A5,A6      - system matrices see below
%          S,s                    - matrices describing constraints on policy
%          JT,J_C1,J_C2,J_C3      - indicator matrix that ensure invertibility of key matrix (regime dependent)
%          W                      - weight matrix for loss function in quadratic form in y'Wy
%          ZZ                     - (n by 1) vector: how many periods are spent in each of the n regimes?
%          RR                     - (n by 1) vector: what type of regime is each of these n regimes? 
%                                   each type is associated with a distinct J matrix
%          H1,H2,H4               - intial guesses for decision rule
%          V_x,V_y,V_yx,V_yc,V_xc - initial guesses for value function of post terminal regime 
%          beta                   - discount factor
%          t                      - tolerance for convergence
%          step                   - step length in updating value function guesses
%
% OUTPUT: 
%          H1S,H2S,H3S,H4S        - stacked decision rules for Y (one set of matrices for every period)
%          F1S,F2S,F3S,F4S        - stacked decision rules for X (one set of matrices for every period)
%          V_xS,V_yS,V_yxS        - stacked value function matrices
%          V_ycS,V_xcS            - stacked value function matrices
%          MUS                    - stacked multiplier ??????
%
% Constraints on policy:
% S*X_t >= s  where S is a matrix picking linear combinations that are bound above by some vector s
%
% Model is of the form:
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X_{t} + A6*X_{t-1} + A5*V_{t}
%
% The solution is of the form: 
% y_t = H1*y_t-1 + H2*x_t-1 + H3*v_t + H4    
% X_t = F1*y_t-1 + F2*x_t-1 + F3*v_t + F4
%
% Value function is of the form
% V_t = Y'_t*V_y*Y_t + 2Y'_t*V_yx*X_t +X'_t*V_xx*X_t +2*Y'_t*V_yc +2*X'_t*V_xc + Sigma

dy  = size(A0,2);  % number of non-policy variables
dx  = size(A3,2);  % number of policy variables
dv  = size(A5,2);  % number of shocks
dmu = size(s,1);   % number of inequality constraints
T   = sum(ZZ,1);   % total number of periods in all regimes but terminal one

% STEP 1: solve for time invariant decision rules after the terminal regime
% --------------------------------------------------------------------------

[V_xt,V_yt,V_yxt,V_yct,V_xct,H1t,H2t,H3t,H4t,F1t,F2t,F3t,F4t,MUst] = ...
solve_discretion_stationary(A0,A1,A2,A3,A5,A6,H1,H2,H4,beta,t,W,V_x,V_y,V_yx,V_yc,V_xc,J_T,S,s,step);

% STEP 2: Use these functions to obtain optimal rules whilst ZLB still binds (a regime characterised by J_C indicator matrix)
% Note we will put the terminal regime in for one period here, hence the  '+1'
H1S = zeros(dy*(T+1),dy); 
H2S = zeros(dy*(T+1),dx);
H3S = zeros(dy*(T+1),dv);
H4S = zeros(dy*(T+1),1);

F1S = zeros(dx*(T+1),dy);
F2S = zeros(dx*(T+1),dx);
F3S = zeros(dx*(T+1),dv);
F4S = zeros(dx*(T+1),1);

VyS  = zeros(dy*(T+1),dy);
VyxS = zeros(dy*(T+1),dx);
VxS  = zeros(dx*(T+1),dx);
VycS = zeros(dy*(T+1),1);
VxcS = zeros(dx*(T+1),1);

MUS = zeros(dmu*(T+1),(dy+dx+dv+1));

H1S(dy*T+1:dy*(T+1),:) = H1t; % We fill up the stacked matrices from the bottom up
H2S(dy*T+1:dy*(T+1),:) = H2t;
H3S(dy*T+1:dy*(T+1),:) = H3t;
H4S(dy*T+1:dy*(T+1),:) = H4t;

F1S(dx*T+1:dx*(T+1),:) = F1t;
F2S(dx*T+1:dx*(T+1),:) = F2t;
F3S(dx*T+1:dx*(T+1),:) = F3t;
F4S(dx*T+1:dx*(T+1),:) = F4t;

VyS(dy*T+1:dy*(T+1),:)  = V_yt;
VyxS(dy*T+1:dy*(T+1),:) = V_yxt;
VxS(dx*T+1:dx*(T+1),:)  = V_xt;
VycS(dy*T+1:dy*(T+1),1) = V_yct;
VxcS(dx*T+1:dx*(T+1),1) = V_xct;

MUS(dmu*T+1:dmu*(T+1),:) = MUst;


% Now start loops filling out stacked transition matrices in event that bounds restrict policy 
last_regime    = size(ZZ,1); 
i              = 1;           % how many periods away from the terminal regime ?


% go backwards in time through regimes and periods
for regime = last_regime : -1 : 1;  
    
    time_elsewhere =  sum( ZZ(regime+1:last_regime,1) );  % Not pretty, but works: for regime = last_regime, sum( ZZ(regime+1:last_regime,1) ) = 0
    
    % Need to use regime-specific indicator matrix
    if     RR(regime,1) == 0;   J = J_T;
    elseif RR(regime,1) == 1;   J = J_C1; 
    elseif RR(regime,1) == 2;   J = J_C2;
    elseif RR(regime,1) == 3;   J = J_C3; end;
        
% in each regime, just figure out optimal setting of instruments and private decision rules in every period.    
while i < time_elsewhere + ZZ(regime,1) + 1
    
    % Takes next period's policy rule and sets the appropriate component equal to H1; subsequent likewise...
    H1 = H1S((dy*(T-i+1)+1):dy*(T-i+1+1),:); 
    H2 = H2S((dy*(T-i+1)+1):dy*(T-i+1+1),:);
    H3 = H3S((dy*(T-i+1)+1):dy*(T-i+1+1),:);
    H4 = H4S((dy*(T-i+1)+1):dy*(T-i+1+1),:);
    
    V_y  =  VyS((dy*(T-i+1)+1):dy*(T-i+1+1),:);
    V_yx = VyxS((dy*(T-i+1)+1):dy*(T-i+1+1),:);
    V_x  =  VxS((dx*(T-i+1)+1):dx*(T-i+1+1),:);
    V_yc = VycS((dy*(T-i+1)+1):dy*(T-i+1+1),1);
    V_xc = VxcS((dx*(T-i+1)+1):dx*(T-i+1+1),1);
    
    % First create DD ('delta') coefficient matrices
    DD1 = (beta*(V_yx')+(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*(W+beta*V_y))*((A0-A2*H1)^(-1))*(A3+A2*H2) + ...
           beta*V_x + beta*(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*V_yx;
    DD2 = (beta*(V_yx')+(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*(W+beta*V_y))*((A0-A2*H1)^(-1))*A1;
    DD3 = (beta*(V_yx')+(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*(W+beta*V_y))*((A0-A2*H1)^(-1))*A6;
    DD4 = (beta*(V_yx')+(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*(W+beta*V_y))*((A0-A2*H1)^(-1))*A5;
    DD5 = (beta*(V_yx')+(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*(W+beta*V_y))*((A0-A2*H1)^(-1))*A2*H4 + ...
           beta*V_xc + beta*(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*V_yc;

    % put together with constraints in matrix MM for x_t and mu_t, with matrix NN on RHS that is premultiplied by inverse of MM
    MM                               = zeros(dmu+dx);
    MM(1:dmu        , 1:dmu)         = eye(dmu)-J;
    MM(1:dmu        , dmu+1: dmu+dx) = J*S;
    MM(dmu+1:dmu+dx , 1:dmu)         = -(S')*J;
    MM(dmu+1:dmu+dx , dmu+1:dmu+dx)  = DD1;

    NN                                  = zeros((dmu+dx),(dy+dx+dv+1));        % Note 'constant' is a vector
    NN(1:dmu        , dy+dx+dv+1)       = J*s;
    NN(dmu+1:dmu+dx , 1:dy)             = -DD2;
    NN(dmu+1:dmu+dx , dy+1:dy+dx)       = -DD3;
    NN(dmu+1:dmu+dx , dy+dx+1:dy+dx+dv) = -DD4;
    NN(dmu+1:dmu+dx , dy+dx+dv+1)       = -DD5;
    
    % Obtain rule for x, hope MM is not ill conditioned ...
    % cond(MM) 
    % size(MM)
    % rank(MM)
    
    PP     = (MM^(-1))*NN;
    F1_new = PP(dmu+1:dmu+dx , 1:dy);             % Coefficient on lagged y...
    F2_new = PP(dmu+1:dmu+dx , dy+1:dy+dx);       % Coefficient on lagged x ... 
    F3_new = PP(dmu+1:dmu+dx , dy+dx+1:dy+dx+dv); % Coefficient on shock 
    F4_new = PP(dmu+1:dmu+dx , dy+dx+dv+1);       % Coefficient on constant
    
    % We'll also want to keep the rule for the multipliers...
    MUs = PP(1:dmu,:);

    % Now convert into rule for y:
    H1_new = ((A0-A2*H1)^(-1))*(A1+(A3+A2*H2)*F1_new);
    H2_new = ((A0-A2*H1)^(-1))*(A6+(A3+A2*H2)*F2_new);
    H3_new = ((A0-A2*H1)^(-1))*(A5+(A3+A2*H2)*F3_new);
    H4_new = ((A0-A2*H1)^(-1))*(A2*H4+(A3+A2*H2)*F4_new);   
    
    % Now get (useful parts of) value function for the ith period before terminal regime:
    V_y_old  = V_y;
    V_yx_old = V_yx;
    V_x_old  = V_x;
    V_yc_old = V_yc;
    V_xc_old = V_xc;

    V_y  = (H1_new')*(W+beta*V_y_old)*H1_new + beta*(H1_new')*V_yx_old*F1_new + beta*(F1_new')*(V_yx_old')*H1_new + beta*(F1_new')*V_x_old*F1_new; % Note we use the rules derived for contemporary period to update, and get value as a function of 
    V_yx = (H1_new')*(W+beta*V_y_old)*H2_new + beta*(H1_new')*V_yx_old*F2_new + beta*(F1_new')*(V_yx_old')*H2_new + beta*(F1_new')*V_x_old*F2_new;
    V_x  = (H2_new')*(W+beta*V_y_old)*H2_new + beta*(H2_new')*V_yx_old*F2_new + beta*(F2_new')*(V_yx_old')*H2_new + beta*(F2_new')*V_x_old*F2_new;
    V_yc = beta*(H1_new')*V_yc_old + beta*(F1_new')*V_xc_old + (H1_new')*(W+beta*V_y_old)*H4_new + beta*(H1_new')*V_yx_old*F4_new + ...
           beta*(F1_new')*(V_yx_old')*H4_new + beta*(F1_new')*V_x_old*F4_new;
    V_xc = beta*(H2_new')*V_yc_old + beta*(F2_new')*V_xc_old + (H2_new')*(W+beta*V_y_old)*H4_new + beta*(H2_new')*V_yx_old*F4_new + ...
           beta*(F2_new')*(V_yx_old')*H4_new + beta*(F2_new')*V_x_old*F4_new;
       
    % check FOC hold ex-post ( this is overkill, worth if some matrix is ill conditioned ? )
    lambda_1  = (A0'-H1'*A2')^(-1)*((W+beta*V_y_old)*H1_new + beta*V_yx_old*F1_new);
    lambda_2  = (A0'-H1'*A2')^(-1)*((W+beta*V_y_old)*H2_new + beta*V_yx_old*F2_new);
    lambda_3  = (A0'-H1'*A2')^(-1)*((W+beta*V_y_old)*H3_new + beta*V_yx_old*F3_new);
    lambda_4  = (A0'-H1'*A2')^(-1)*((W+beta*V_y_old)*H4_new + beta*V_yx_old*F4_new  + beta*V_yc_old);    
    mu_1      = MUs(:,1:dy);
    mu_2      = MUs(:,dy+1:dy+dx);
    mu_3      = MUs(:,dy+dx+1:dy+dx+dv);
    mu_4      = MUs(:,dy+dx+dv+1:dy+dx+dv+1);
    check_y_y = (W + beta*V_y_old)*H1_new + beta*V_yx_old*F1_new - (A0'-H1'*A2')*lambda_1;
    check_y_x = (W + beta*V_y_old)*H2_new + beta*V_yx_old*F2_new - (A0'-H1'*A2')*lambda_2;
    check_y_v = (W + beta*V_y_old)*H3_new + beta*V_yx_old*F3_new - (A0'-H1'*A2')*lambda_3;
    check_y_c = (W + beta*V_y_old)*H4_new + beta*V_yx_old*F4_new - (A0'-H1'*A2')*lambda_4 + beta*V_yc_old;
    check_x_y = beta*V_yx_old'*H1_new + beta*V_x_old*F1_new + (A3'+ H2'*A2')*lambda_1 - S'*mu_1;
    check_x_x = beta*V_yx_old'*H2_new + beta*V_x_old*F2_new + (A3'+ H2'*A2')*lambda_2 - S'*mu_2;
    check_x_v = beta*V_yx_old'*H3_new + beta*V_x_old*F3_new + (A3'+ H2'*A2')*lambda_3 - S'*mu_3;
    check_x_c = beta*V_yx_old'*H4_new + beta*V_x_old*F4_new + beta*V_xc_old + (A3'+ H2'*A2')*lambda_4 - S'*mu_4;   
    check_l_y = (A0-A2*H1)*H1_new - (A3+A2*H2)*F1_new - A1; 
    check_l_x = (A0-A2*H1)*H2_new - (A3+A2*H2)*F2_new - A6; 
    check_l_v = (A0-A2*H1)*H3_new - (A3+A2*H2)*F3_new - A5; 
    check_l_c = (A0-A2*H1)*H4_new - (A3+A2*H2)*F4_new - A2*H4;
    
    max_dev = [ max(max(abs(check_y_y))),max(max(abs(check_y_x))),max(max(abs(check_y_v))),max(max(abs(check_y_c)));...
                max(max(abs(check_x_y))),max(max(abs(check_x_x))),max(max(abs(check_x_v))),max(max(abs(check_x_c)));...
                max(max(abs(check_l_y))),max(max(abs(check_l_x))),max(max(abs(check_l_v))),max(max(abs(check_l_c)))];
    
 %   fprintf('\n Maximum deviation from FOC holding is %g \n', max(max(max_dev)));
        
    % Now enter into stacked matrix
    H1S((dy*(T-i)+1):dy*(T-i+1),:) = H1_new;
    H2S((dy*(T-i)+1):dy*(T-i+1),:) = H2_new;
    H3S((dy*(T-i)+1):dy*(T-i+1),:) = H3_new;
    H4S((dy*(T-i)+1):dy*(T-i+1),:) = H4_new;

    F1S((dx*(T-i)+1):dx*(T-i+1),:) = F1_new;
    F2S((dx*(T-i)+1):dx*(T-i+1),:) = F2_new;
    F3S((dx*(T-i)+1):dx*(T-i+1),:) = F3_new;
    F4S((dx*(T-i)+1):dx*(T-i+1),:) = F4_new;

    VyS((dy*(T-i)+1):dy*(T-i+1),:)  = V_y;
    VyxS((dy*(T-i)+1):dy*(T-i+1),:) = V_yx;
    VxS((dx*(T-i)+1):dx*(T-i+1),:)  = V_x;
    VycS((dy*(T-i)+1):dy*(T-i+1),1) = V_yc;
    VxcS((dx*(T-i)+1):dx*(T-i+1),1) = V_xc;
    
    MUS((dmu*(T-i)+1):(dmu*(T-i+1)),:) = MUs;
    
    % move one more period backwards in time
    i = i+1;
end

end;