function [IRFs,P0] = solve_committment_simple(A0,A1,A2,A3,A5,A6,beta,W,ZZ,RR,J0,J1,T_IRF,u1,s,S); 

% INPUT:   
%          A0,A1,A2,A3,A5,A6      - system matrices see below
%          beta                   - discount factor
%          W                      - weight matrix for loss function in quadratic form in y'Wy
%          ZZ                     - (k,1) vector, each entry gives number of periods in each of k regimes
%          RR                     - (k,1) vector, each entry gives type of constraints in each of k regimes
%                                    for every type, there is one J - matrix
%          J1,J2                  - indicator matrix that ensure invertibility of key matrix
%          T_IRF                  - length of periods for impulse responses
%          S,s                    - matrices describing constraints on policy
% 
% OUTPUT: IRFs                    - impulse responses: 1. lagrange multipliers, 2. non-policy variables, 3. policy variables
%         P0                      - transition matrix in post-terminal regime
%
% Constraints on policy:
% S*X_t >= s  where S is a matrix picking linear combinations that are bound above by some vector s
%
% Model is of the form:
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X_{t} + A6*X_{t-1} + A5*V_{t}
%
%
% Lagrangean of the policy problem
% ---------------------------------
% sum[beta^t Y_t'*W*Y_t + 2*lambda_t*(A0*Y_t-A1*Y_{t-1}-A2*Y_{t+1}-A3*X_t-A6*X_{t-1}-A5*v_t) + 2*mu_t(S*x_t-s)]  
% FOC for mu_t      0 = mu_t*(S*x_t-s)    (Kuhn Tacker representaton)
% FOC for Y_t:      0 = W*Y_t + A0'*lambda_t -1/beta*A2'*lambda_{t-1} - beta*A1'*lambda_{t+1}
% FOC for lambda_t  0 = A0*Y_t-A1*Y_{t-1}-A2*Y_{t+1}-A3*X_t+A6*X_{t-1}+A5*v_t
% FOC for X_t:      0 = -A3'*lambda_t - beta*A6'*lambda_{t-1} + S'*mu_t
%
% Dynamic system: H*Z_{t+1} + G_{k}*Z_t + F*Z_{t-1} = Psi_{k}*u_t   
%           with: Z = [mu_t,lambda_t,Y_t,X_t] and u_t = [v_t,s] 


dy  = size(A1,1);   % number of non-policy variables
dl  = dy;           % number of Lagrange multipliers on nonpolicy variables
dx  = size(A3,2);   % number of policy instruments
dm  = dx;           % number of lagrange multipliers on policy variables
dv  = size(A5,2);   % number of shocks
dim = 2*(dx+dy);    % total number of variables, including multipliers but excluding i.i.d shocks


G0 = zeros(dim);
G0(1:dm,1:dm)                                  = eye(dm)-J0;
G0(1:dm,(dm+dl+dy+1):(dm+dl+dy+dx))            = J0*S;
G0((dm+1):(dm+dl),(dm+dl+1):(dm+dl+dy))        = A0;
G0((dm+1):(dm+dl),(dm+dl+dy+1):(dm+dl+dy+dx))  = -A3;
G0((dm+dl+1):(dm+dl+dy),(dm+1):(dm+dl))        = A0';
G0((dm+dl+1):(dm+dl+dy),(dm+dl+1):(dm+dl+dy))  = W;
G0((dm+dl+dy+1):(dm+dl+dy+dx),1:dm)            = (J0*S)';
G0((dm+dl+dy+1):(dm+dl+dy+dx),(dm+1):(dm+dl))  = -A3';

F = zeros(dim);
F((dm+1):(dm+dl),(dm+dl+1):(dm+dl+dy))       = -A1;
F((dm+1):(dm+dl),(dm+dl+dy+1):(dm+dl+dy+dx)) = -A6;
F((dm+dl+1):(dm+dl+dy),(dm+1):(dm+dl))       = -(1/beta)*(A2');

H = zeros(dim);
H((dm+1):(dm+dl),(dm+dl+1):(dm+dl+dy))       = -A2;
H((dm+dl+1):(dm+dl+dy),(dm+1):(dm+dl))       = -beta*(A1');
H((dm+dl+dy+1):(dm+dl+dy+dx),(dm+1):(dm+dl)) = -beta*(A6');

Psi0 = zeros(dim,(dv+dm));
Psi0(1:dm,(dv+1):(dv+dm)) = J0;
Psi0((dm+1):(dm+dl),1:dv) = A5;
  

% set up the regime dependent G{k}, Psi{k} matrices
% -------------------------------------------------

G1 = G0;
G1(1:dm,1:dm)                       = eye(dm)-J1;
G1(1:dm,(dm+dl+dy+1):(dm+dl+dy+dx)) = J1*S;
G1((dm+dl+dy+1):(dm+dl+dy+dx),1:dm) = (J1*S)';


Psi1 = Psi0;
Psi1(1:dm,(dv+1):(dv+dm)) = J1;

    

% Now fill out stacked time matrices
% ------------------------------------
total_periods  = sum(ZZ);     % total number of periods in all pre-terminal regimes
total_regimes  = size(ZZ,1);  % total number of pre-termimal regimes
p_else         = 0;           % how many periods have been spent in other regimes?

J = zeros((total_periods+1)*dm);
M = zeros((total_periods+1)*dim,1);

i = 1;  % counter for periods

for regime = 1 : total_regimes;  
       
    p_here = ZZ(regime,1);
    
    if     RR(regime,1) == 1;        G = G0; Psi = Psi0;
    elseif RR(regime,1) == 2;        G = G1; Psi = Psi1;
    end;
    while i < p_else + p_here + 1;
            J(dim*(i-1)+1:dim*(i-1)+dim,dim*(i-1)+1:dim*(i-1)+dim) = G; 
            J(dim*(i-1)+1:dim*(i-1)+dim,dim*i+1:dim*i+dim)         = H;
            J(dim*i+1:dim*i+dim,dim*(i-1)+1:dim*(i-1)+dim)         = F;         
            if i == 1;
                 M(dim*(i-1)+1:dim*(i-1)+dim,1)                    = Psi*u1;                   % initial shock as specified by u1
            else M(dim*(i-1)+1:dim*(i-1)+dim,1)                    = Psi*[zeros(dv,1); s];     % no more shocks thereafter
            end;
            i = i+1;
    end
    p_else = p_else + p_here;
end;
    

% Now fill in the terminal regime's transition matrices
%-------------------------------------------------------
  
% Now put terminal system into gensys format to solve for reduced-form transition equation in that regime
g0  = [[G0, H];[eye(dim), zeros(dim)]];          % coefficients on current period variables (including expectations of tomorrows variables as separate variables)
g1	= [[-F,zeros(dim)];[zeros(dim),eye(dim)]];   % coefficients on last period variables
c   = zeros(2*dim,1);                            % constant (not present in terminal regime)
psi = [Psi0(:,1:dv);zeros(dim,dv)];              % coefficients on structural shocks (taking out the constant part of the Psi0 matrix...)
pie = [zeros(dim);eye(dim)];                     % coefficients on expectational errors
div = 1;                                         % cutoff between explosive and non-explosive roots

addpath([cd,'\gensys']);
[NA,c,NB,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,c,psi,pie,div);
rmpath([cd,'\gensys']);

%eu; Eigenvalue info

% Extract transition matrix (discard expectational variables that are of no interest here)
P0 = NA(1:dim,1:dim);
N0 = NB(1:dim,1:dv);


max_eig = max(abs(eig(P0)));

%fprintf('\n Maximum eigenvalue of the system = %g \n',max_eig);


% Puts in 'terminal condition' matrix in final diagonal
%------------------------------------------------------    
J(dim*p_else+1:dim*p_else+dim , dim*p_else+1:dim*p_else+dim) = G0 + H*P0;                   % Terminal entry in J matrix, in Regime 0
M(dim*p_else+1:dim*p_else+dim ,1)                            = Psi0*[zeros(dv,1); s];       % Terminal entry in M vector, in Regime 0

% solve the system
%-----------------
Z = J\M;


% compute impulse responses
% --------------------------
IRFs = zeros(dim,T_IRF); % Use to store response to specified shocks


if total_periods > 0;   % lower bound binding for a least some periods
    i = 1;
    while i < total_periods +1;
        IRFs(:,i) = Z(dim*(i-1)+1:dim*(i-1)+dim,1);
        i = i+1;
    end

    % from here just use transiton matrices from post terminal regime
    while i < T_IRF + 1
        IRFs(:,i) = P0*IRFs(:,i-1);
        i = i+1;
    end
else  % lower bound never binding
    
IRFs(:,1) = N0*u1(1:dv,1);
i = 2;
% from here just use transiton matrices from post terminal regime
    while i < T_IRF
        IRFs(:,i) = P0*IRFs(:,i-1);
        i = i+1;
    end 
end
    