function [V_x,V_y,V_yx,V_yc,V_xc,H1,H2,H3,H4,F1,F2,F3,F4,MUs] = solve_discretion_stationary(A0,A1,A2,A3,A5,A6,H1,H2,H4,beta,t,W,V_x,V_y,V_yx,V_yc,V_xc,J_T,S,s, step)

% INPUT:   
%          A0,A1,A2,A3,A5,A6      - system matrices see below
%          H1,H2,H4               - intial guesses for decision rule
%          beta                   - discount factor
%          t                      - tolerance for convergence
%          W                      - weight matrix for loss function in quadratic form in y'Wy
%          V_x,V_y,V_yx,V_yc,V_xc - initial guesses for value function
%          JT                     - indicator matrix that ensure invertibility of key matrix
%          S,s                    - matrices describing constraints on policy
%          step                   - step length (in 0 to 1) in updating guesses : 1 full update, 0 no update.
% 
% OUTPUT: 
%          H1,H2,H4               - decision rules for Y
%          F1, F2, F3, F4         - decision rules for X
%          V_x,V_y,V_yx,V_yc,V_xc - value function matrices
%
% Constraints on policy:
% S*X_t >= s  where S is a matrix picking linear combinations that are bound above by some vector s
%
% Model is of the form:
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X_{t} + A6*X_{t-1} + A5*V_{t}
%
% The solution is of the form: 
% y_t = H1*y_t-1 + H2*x_t-1 + H3*v_t + H4    (H3 is not solved for because we focus on perfect foresight)
% X_t = F1*y_t-1 + F2*x_t-1 + F3*v_t + F4
%
% Value function is of the form
% V_t = Y'_t*V_y*Y_t + 2Y'_t*V_yx*X_t +X'_t*V_xx*X_t +2*Y'_t*V_yc +2*X'_t*V_xc + Sigma
% 

dy  = size(A0,2);  % number of non-policy variables
dx  = size(A3,2);  % number of policy variables
dv  = size(A5,2);  % number of shocks
dmu = size(s,1);   % number of inequality constraints


% Put together constraints in matrix for x_t and mu_t, called MM, with matrix NN on RHS that is premultiplied by inverse of MM
MM                         = zeros(dmu+dx);
MM(1:dmu,1:dmu)            = eye(dmu)-J_T;
MM(1:dmu,(dmu+1):(dmu+dx)) = J_T*S; 
MM((dmu+1):(dmu+dx),1:dmu) = -(S')*J_T; 
NN                         = zeros((dmu+dx),(dy+dx+dv+1)); % Note 'constant' is a vector
NN(1:dmu,(dy+dx+dv+1))     = J_T*s; 

    
i = 9999;  j = i;

while max(i,j) > t

    H1_old = H1;
    H2_old = H2;
    H4_old = H4;
    
    V_y_old  = V_y;
    V_yx_old = V_yx;
    V_x_old  = V_x;
    V_yc_old = V_yc;
    V_xc_old = V_xc;
    
    % Step 1: Get evolution of x_t in a form that can be used to extract a policy rule
    % First create DD ('delta') coefficient matrices
    cc  = (beta*(V_yx')+(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*(W+beta*V_y))*((A0-A2*H1)^(-1));
    DD1 = cc*(A3+A2*H2) + beta*V_x + beta*(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*V_yx;
    DD2 = cc*A1;
    DD3 = cc*A6;
    DD4 = cc*A5;
    DD5 = cc*A2*H4 + beta*V_xc + beta*(A3'+(H2')*(A2'))*((A0'-(H1')*(A2'))^(-1))*V_yc;

    % Put together the matrices for constraints
    MM((dmu+1):(dmu+dx),(dmu+1):(dmu+dx))     =  DD1;    
    NN((dmu+1):(dmu+dx),1:dy)                 = -DD2;
    NN((dmu+1):(dmu+dx),(dy+1):(dy+dx))       = -DD3;
    NN((dmu+1):(dmu+dx),(dy+dx+1):(dy+dx+dv)) = -DD4;
    NN((dmu+1):(dmu+dx),(dy+dx+dv+1))         = -DD5;
    
    % cond(MM)
        
    % Obtain rule for x...
    PP = (MM^(-1))*NN;
    F1 = PP((dmu+1):(dmu+dx),1:dy);                  % Coefficient on lagged y...
    F2 = PP((dmu+1):(dmu+dx),(dy+1):(dy+dx));        % on lagged x ... etc
    F4 = PP((dmu+1):(dmu+dx),(dy+dx+dv+1));
            
    % Now update rule for y:
    H1 = ((A0-A2*H1_old)^(-1))*(A1+(A3+A2*H2_old)*F1);
    H2 = ((A0-A2*H1_old)^(-1))*(A6+(A3+A2*H2_old)*F2);
    H4 = ((A0-A2*H1_old)^(-1))*(A2*H4_old+(A3+A2*H2_old)*F4);
        
    % Step 2: Update value functions (ignoring constant term)
    % Note we use the rules derived for contemporary period to update, and get value as a function of 
    V_y  = (H1')*(W+beta*V_y_old)*H1 + beta*(H1')*V_yx_old*F1 + beta*(F1')*(V_yx_old')*H1 + beta*(F1')*V_x_old*F1; 
    V_yx = (H1')*(W+beta*V_y_old)*H2 + beta*(H1')*V_yx_old*F2 + beta*(F1')*(V_yx_old')*H2 + beta*(F1')*V_x_old*F2;
    V_x  = (H2')*(W+beta*V_y_old)*H2 + beta*(H2')*V_yx_old*F2 + beta*(F2')*(V_yx_old')*H2 + beta*(F2')*V_x_old*F2;
    V_yc =  beta*(H1')*V_yc_old + beta*(F1')*V_xc_old + (H1')*(W+beta*V_y_old)*H4 + beta*(H1')*V_yx_old*F4 + ... 
            beta*(F1')*(V_yx_old')*H4 + beta*(F1')*V_x_old*F4;
    V_xc =  beta*(H2')*V_yc_old + beta*(F2')*V_xc_old + (H2')*(W+beta*V_y_old)*H4 + beta*(H2')*V_yx_old*F4 + ... 
            beta*(F2')*(V_yx_old')*H4 + beta*(F2')*V_x_old*F4;
        
    
    H1 =  step*H1 +(1-step)*H1_old;
    H2 =  step*H2 +(1-step)*H2_old;
    H4 =  step*H4 +(1-step)*H4_old;
    
    V_y  = step*V_y  + (1-step)*V_y_old;
    V_yx = step*V_yx + (1-step)*V_yx_old;
    V_x  = step*V_x  + (1-step)*V_x_old;
    V_yc = step*V_yc + (1-step)*V_yc_old;
    V_xc = step*V_xc + (1-step)*V_xc_old;
        
    % Take biggest differences between matrices...
    i = max([max(abs(H1-H1_old))  , max(abs(H2-H2_old))    , max(abs(H4-H4_old))]);
    j = max([max(abs(V_y-V_y_old)), max(abs(V_yx-V_yx_old)), max(abs(V_x-V_x_old)), max(abs(V_yc-V_yc_old)), max(abs(V_xc-V_xc_old))]);
end

% These objects can be computed after convergence is achieved
MUs = PP(1:dmu,:);
F3  = PP((dmu+1):(dmu+dx),(dy+dx+1):(dy+dx+dv));
H3  = ((A0-A2*H1)^(-1))*(A5+(A3+A2*H2)*F3);


 % check FOC hold ex-post ( this is overkill, worth if some matrix is ill conditioned ? )
    lambda_1  = (A0'-H1'*A2')^(-1)*((W+beta*V_y)*H1 + beta*V_yx*F1);
    lambda_2  = (A0'-H1'*A2')^(-1)*((W+beta*V_y)*H2 + beta*V_yx*F2);
    lambda_3  = (A0'-H1'*A2')^(-1)*((W+beta*V_y)*H3 + beta*V_yx*F3);
    lambda_4  = (A0'-H1'*A2')^(-1)*((W+beta*V_y)*H4 + beta*V_yx*F4  + beta*V_yc);    
    mu_1      = MUs(:,1:dy);
    mu_2      = MUs(:,dy+1:dy+dx);
    mu_3      = MUs(:,dy+dx+1:dy+dx+dv);
    mu_4      = MUs(:,dy+dx+dv+1:dy+dx+dv+1);
    check_y_y = (W + beta*V_y)*H1 + beta*V_yx*F1 - (A0'-H1'*A2')*lambda_1;
    check_y_x = (W + beta*V_y)*H2 + beta*V_yx*F2 - (A0'-H1'*A2')*lambda_2;
    check_y_v = (W + beta*V_y)*H3 + beta*V_yx*F3 - (A0'-H1'*A2')*lambda_3;
    check_y_c = (W + beta*V_y)*H4 + beta*V_yx*F4 - (A0'-H1'*A2')*lambda_4 + beta*V_yc_old;
    check_x_y = beta*V_yx'*H1 + beta*V_x*F1 + (A3'+ H2'*A2')*lambda_1 - S'*mu_1;
    check_x_x = beta*V_yx'*H2 + beta*V_x*F2 + (A3'+ H2'*A2')*lambda_2 - S'*mu_2;
    check_x_v = beta*V_yx'*H3 + beta*V_x*F3 + (A3'+ H2'*A2')*lambda_3 - S'*mu_3;
    check_x_c = beta*V_yx'*H4 + beta*V_x*F4 + beta*V_xc + (A3'+ H2'*A2')*lambda_4 - S'*mu_4;   
    check_l_y = (A0-A2*H1)*H1 - (A3+A2*H2)*F1 - A1; 
    check_l_x = (A0-A2*H1)*H2 - (A3+A2*H2)*F2 - A6; 
    check_l_v = (A0-A2*H1)*H3 - (A3+A2*H2)*F3 - A5; 
    check_l_c = (A0-A2*H1)*H4 - (A3+A2*H2)*F4 - A2*H4;
    
    max_dev = [ max(max(abs(check_y_y))),max(max(abs(check_y_x))),max(max(abs(check_y_v))),max(max(abs(check_y_c)));...
                max(max(abs(check_x_y))),max(max(abs(check_x_x))),max(max(abs(check_x_v))),max(max(abs(check_x_c)));...
                max(max(abs(check_l_y))),max(max(abs(check_l_x))),max(max(abs(check_l_v))),max(max(abs(check_l_c)))];    
    
%    fprintf('\n Maximum deviation from FOC holding in stationary equilibrium is %g \n', max(max(max_dev)));